/**
 * Vue Number Format 1.0.0
 * (c) 2018-2021 Dipak Sarkar <hello@dipaksarkar.in> (https://dipaksarkar.in/)
 * @license MIT
 */
'use strict';Object.defineProperty(exports,'__esModule',{value:true});var options = {
  prefix: '',
  suffix: '',
  separator: ',',
  decimal: '.',
  precision: 2,
  prefill: true,
  reverseFill: false,
  min: false,
  max: false,
  null_value: ''
};/**
 * Number format function
 * @param {Object} options
 */
function NumberFormat(config) {
  var this$1$1 = this;
  if ( config === void 0 ) config = options;

  this.options = Object.assign(options, config);
  this.input = '';
  this.number = '';
  this.isClean = false;

  this.isNull = function (input) {
    if ( input === void 0 ) input = this$1$1.input;

    return !this$1$1.numberOnly(input, new RegExp('[^0-9]+', 'gi'));
  };

  this.clean = function (clean) {
    if ( clean === void 0 ) clean = false;

    this$1$1.isClean = clean;
    return this$1$1
  };

  this.sign = function () {
    var sign = (this$1$1.input.toString().indexOf('-') >= 0 && this$1$1.realNumber() > 0) ? '-' : '';
    return sign
  };

  function between(min, n, max) {
    return Math.max(min, Math.min(n, max))
  }

  // Uncaught RangeError: toFixed() digits argument must be between 0 and 20 at Number.toFixed
  function fixed(precision) {
    return between(0, precision, 20)
  }

  function toFixed(numbers, precision) {
    // eslint-disable-next-line no-restricted-properties
    var exp = Math.pow(10, precision);
    var float = parseFloat(numbers) / exp;
    return float.toFixed(fixed(precision))
  }

  this.toNumber = function (string) { return Number(string); };

  this.numberOnly = function (string, regExp) { return string.toString().replace(regExp, ''); };

  this.isNegative = this.sign() === '-';

  this.numbers = function () {
    if (this$1$1.options.reverseFill) {
      this$1$1.number = toFixed(this$1$1.numberOnly(this$1$1.input, /\D+/g), this$1$1.options.precision).replace('.', this$1$1.options.decimal);
    } else if (typeof this$1$1.input === 'number') {
      if (this$1$1.isClean) {
        this$1$1.number = this$1$1.toNumber(this$1$1.input.toFixed(this$1$1.options.precision)).toString().replace('-', '').replace('.', this$1$1.options.decimal);
      } else {
        this$1$1.number = this$1$1.toNumber(this$1$1.input).toString().replace('-', '').replace('.', this$1$1.options.decimal);
      }
    } else {
      this$1$1.number = this$1$1.numberOnly(this$1$1.input, new RegExp(("[^0-9\\" + (this$1$1.options.decimal) + "]+"), 'gi'));
      this$1$1.number = this$1$1.parts(this$1$1.number).join(this$1$1.options.decimal);
    }
    return this$1$1.number
  };

  this.realNumber = function () { return this$1$1.toNumber(this$1$1.numbers().toString().replace(this$1$1.options.decimal, '.')); };

  this.parts = function (number, decimal) {
    if ( number === void 0 ) number = '';
    if ( decimal === void 0 ) decimal = this$1$1.options.decimal;

    var parts = number.toString().split(decimal);
    parts[0] = this$1$1.toNumber(parts[0]) || 0;
    if (parts.length > 1) {
      parts[1] = parts.slice(1, parts.length).join('');
      parts = parts.slice(0, 2);
      if (this$1$1.isClean && parts[1].length > this$1$1.options.precision) {
        parts[1] = this$1$1.toNumber(("." + (parts[1]))).toFixed(this$1$1.options.precision).toString().replace('0.', '');
      }
    }
    return parts.slice(0, 2)
  };

  this.addSeparator = function () {
    var parts = this$1$1.numbers().split(this$1$1.options.decimal);
    parts[0] = parts[0].toString().replace(/(\d)(?=(?:\d{3})+\b)/gm, ("$1" + (this$1$1.options.separator)));
    if (this$1$1.isClean) {
      parts[1] = this$1$1.toNumber(("." + (parts[1]))).toString().replace('0.', '');
      return parts[1] && parts[1] >= 0 ? parts.join(this$1$1.options.decimal) : parts[0]
    }
    return parts.join(this$1$1.options.decimal)
  };

  /**
   * Format the input with default config if there is no constructor config
   * @param {Number, String} input
   * @return {String}
   */
  this.format = function (input) {
    if (input === '') { return this$1$1.options.null_value }
    this$1$1.input = input || this$1$1.options.null_value;
    if (this$1$1.isNull()) { return this$1$1.options.null_value }
    return this$1$1.sign() + this$1$1.options.prefix + this$1$1.addSeparator() + this$1$1.options.suffix
  };

  /**
   * Unformat the input with default config if there is no constructor config
   * @param {Number, String} input
   * @return {String}
   */
  this.unformat = function (input) {
    if (input === '') { return this$1$1.options.null_value }
    this$1$1.input = input || this$1$1.options.null_value;
    if (this$1$1.isNull()) { return this$1$1.options.null_value }
    return this$1$1.toNumber(this$1$1.sign() + this$1$1.realNumber())
  };
}// import options from './options'

var CONFIG_KEY$1 = '__input-facade__';

/**
 * Creates a CustomEvent('input') with detail = { facade: true }
 * used as a way to identify our own input event
 */
function FacadeInputEvent() {
  return new CustomEvent('input', {
    bubbles: true,
    cancelable: true,
    detail: { facade: true }
  })
}

/**
 * ensure that the element we're attaching to is an input element
 * if not try to find an input element in this elements childrens
 *
 * @param {HTMLInputElement} el
 */
function getInputElement(el) {
  var inputElement = el instanceof HTMLInputElement ? el : el.querySelector('input');

  /* istanbul ignore next */
  if (!inputElement) {
    throw new Error('facade directive requires an input element')
  }

  return inputElement
}

/**
 * Updates the cursor position to the right place after the masking rule was applied
 * @param {HTMLElement} el
 * @param {Number} position
 */
function updateCursor(el, position) {
  var setSelectionRange = function () { el.setSelectionRange(position, position); };
  setSelectionRange();
  // Android Fix
  setTimeout(setSelectionRange(), 0);
}

/**
 * Updates the element's value and unmasked value based on the masking config rules
 *
 * @param {HTMLInputElement} el The input element to update
 * @param {object} [options]
 * @param {Boolean} options.emit Wether to dispatch a new InputEvent or not
 * @param {Boolean} options.force Forces the update even if the old value and the new value are the same
 */
function updateValue(el, vnode, ref) {
  if ( ref === void 0 ) ref = {};
  var emit = ref.emit; if ( emit === void 0 ) emit = true;
  var force = ref.force; if ( force === void 0 ) force = false;
  var clean = ref.clean; if ( clean === void 0 ) clean = false;

  var ref$1 = el[CONFIG_KEY$1];
  var config = ref$1.config;
  var ref$2 = el[CONFIG_KEY$1];
  var oldValue = ref$2.oldValue;

  var currentValue = vnode && vnode.data.model ? vnode.data.model.value : el.value;

  oldValue = oldValue || '';
  currentValue = currentValue || '';

  var number = new NumberFormat(config).clean(clean);
  var masked = number.format(currentValue);
  var unmasked = number.clean(true).unformat(currentValue);

  // check value with in range max and min value
  if (clean) {
    if (config.max && unmasked > config.max) {
      masked = number.format(config.max);
      unmasked = number.unformat(config.max);
    } else if (config.min && unmasked < config.min) {
      masked = number.format(config.min);
      unmasked = number.unformat(config.min);
    }
  }

  if (force || oldValue !== currentValue) {
    el[CONFIG_KEY$1].oldValue = masked;
    el.unmaskedValue = unmasked;
    // safari makes the cursor jump to the end if el.value gets assign even if to the same value
    if (el.value !== masked) {
      el.value = masked;
    }

    // this part needs to be outside the above IF statement for vuetify in firefox
    // drawback is that we endup with two's input events in firefox
    return emit && el.dispatchEvent(FacadeInputEvent())
  }
}

/**
 * Input event handler
 *
 * @param {Event} event The event object
 */
function inputHandler(event) {
  var target = event.target;
  var detail = event.detail;
  // We dont need to run this method on the event we emit (prevent event loop)
  if (detail && detail.facade) {
    return false
  }

  // since we will be emitting our own custom input event
  // we can stop propagation of this native event
  event.stopPropagation();

  var positionFromEnd = target.value.length - target.selectionEnd;
  var ref = target[CONFIG_KEY$1];
  var oldValue = ref.oldValue;
  var config = ref.config;

  updateValue(target, null, { emit: false });
  // updated cursor position
  positionFromEnd = Math.max(positionFromEnd, config.suffix.length);
  positionFromEnd = target.value.length - positionFromEnd;
  positionFromEnd = Math.max(positionFromEnd, config.prefix.length + 1);
  updateCursor(target, positionFromEnd);

  if (oldValue !== target.value) {
    target.dispatchEvent(FacadeInputEvent());
  }
}

/**
 * Blur event handler
 *
 * @param {Event} event The event object
 */
function blurHandler(event) {
  var target = event.target;
  var detail = event.detail;
  // We dont need to run this method on the event we emit (prevent event loop)
  if (detail && detail.facade) {
    return false
  }

  var ref = target[CONFIG_KEY$1];
  var oldValue = ref.oldValue;

  updateValue(target, null, { force: true, clean: true });

  if (oldValue !== target.value) {
    target.dispatchEvent(FacadeInputEvent());
  }
}/* eslint-disable prefer-object-spread */

var CONFIG_KEY = CONFIG_KEY$1;

var VNumber = {
  bind: function (el, ref, vnode) {
    var value = ref.value;
    var modifiers = ref.modifiers;

    el = getInputElement(el);
    var config = Object.assign({}, options, value, modifiers);
    el[CONFIG_KEY] = { config: config };
    // set initial value
    updateValue(el, vnode, { force: config.prefill, clean: true });
  },

  inserted: function (el) {
    el = getInputElement(el);
    var option = el[CONFIG_KEY];
    var config = option.config;
    // prefer adding event listener to parent element to avoid Firefox bug which does not
    // execute `useCapture: true` event handlers before non-capturing event handlers
    var handlerOwner = el.parentElement || el;

    // use anonymous event handler to avoid inadvertently removing masking for all inputs within a container
    var oninput = function (e) { return inputHandler(e); };

    handlerOwner.addEventListener('input', oninput, true);

    el.onblur = function (e) { return blurHandler(e); };

    // check decimal key and insert to current element
    // updated cursor position after format the value
    el.onkeydown = function (e) {
      if (([110, 190].includes(e.keyCode) || e.key === config.decimal) && !el.value.includes(config.decimal)) {
        e.preventDefault();
        el.setRangeText(config.decimal);
        el.dispatchEvent(new Event('input'));
        updateCursor(el, el.value.indexOf(config.decimal) + 1);
      } else if (([110, 190].includes(e.keyCode) || e.key === config.decimal) && el.value.includes(config.decimal)) {
        e.preventDefault();
      }
    };

    option.cleanup = function () { return handlerOwner.removeEventListener('input', oninput, true); };
  },

  update: function (el, ref, vnode) {
    var value = ref.value;
    var oldValue = ref.oldValue;
    var modifiers = ref.modifiers;

    el = getInputElement(el);
    if (value !== oldValue) {
      var ref$1 = el[CONFIG_KEY];
      var config = ref$1.config;
      el[CONFIG_KEY].config = Object.assign({}, config, value, modifiers);
      updateValue(el, vnode, { force: true, clean: true });
    } else {
      updateValue(el, vnode);
    }
  },

  unbind: function (el) {
    getInputElement(el)[CONFIG_KEY].cleanup();
  }
};//

var script = {
  props: {
    value: {
      required: true,
      type: [Number, String]
    },
    null_value: {
      type: [Number, String],
      default: function () { return options.null_value; }
    },
    masked: {
      type: Boolean,
      default: false
    },
    reverseFill: {
      type: Boolean,
      default: options.reverseFill
    },
    precision: {
      type: Number,
      default: function () { return options.precision; }
    },
    decimal: {
      type: String,
      default: function () { return options.decimal; }
    },
    separator: {
      type: String,
      default: function () { return options.separator; }
    },
    prefix: {
      type: String,
      default: function () { return options.prefix; }
    },
    suffix: {
      type: String,
      default: function () { return options.suffix; }
    }
  },
  directives: {
    number: VNumber
  },
  data: function data() {
    return {
      maskedValue: this.value,
      unmaskedValue: null
    }
  },
  methods: {
    input: function input(ref) {
      var target = ref.target;

      this.maskedValue = target.value;
      this.unmaskedValue = target.unmaskedValue;
      this.$emit('input', this.emittedValue);
    },
    change: function change() {
      this.$emit('change', this.emittedValue);
    }
  },
  computed: {
    emittedValue: function emittedValue() {
      return this.masked ? this.maskedValue : this.unmaskedValue
    },
    config: function config() {
      return this.$props
    }
  }
};function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    var options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    var hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            var originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            var existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}/* script */
var __vue_script__ = script;

/* template */
var __vue_render__ = function () {var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('input',{directives:[{name:"number",rawName:"v-number",value:(_vm.config),expression:"config"}],staticClass:"v-number",attrs:{"type":"text","autocomplete":"off"},domProps:{"value":_vm.maskedValue},on:{"change":_vm.change,"input":_vm.input}},[])};
var __vue_staticRenderFns__ = [];

  /* style */
  var __vue_inject_styles__ = undefined;
  /* scoped */
  var __vue_scope_id__ = undefined;
  /* module identifier */
  var __vue_module_identifier__ = "data-v-544538c6";
  /* functional template */
  var __vue_is_functional_template__ = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  var __vue_component__ = /*#__PURE__*/normalizeComponent(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    false,
    undefined,
    undefined,
    undefined
  );function install(Vue, globalOptions) {
  if (globalOptions) {
    Object.assign(options, globalOptions);
  }
  Vue.directive('number', VNumber);
  Vue.component('number', __vue_component__);
}exports.Number=__vue_component__;exports.VNumber=VNumber;exports['default']=install;exports.options=options;//# sourceMappingURL=index.cjs.js.map
